
extern "C"
{
#include <glib-object.h>
#include <gdk/gdkenumtypes.h>
}

#include <gdkmm/gc.h>
#include <gdkmm/private/gc_p.h>

#include <gdk/gdkenumtypes.h>
// -*- c++ -*-
/* $Id: gc.ccg,v 1.4 2002/04/09 20:53:26 murrayc Exp $ */

/*
 *
 * Copyright 1998-1999 The Gtk-- Development Team
 * Copyright 2001      Free Software Foundation
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <gdk/gdkgc.h>
#include <gdkmm/color.h>
//#include <gdkmm/font.h>
//#include <gdkmm/pixmap.h>
#include <gdkmm/bitmap.h>
#include <gdkmm/colormap.h>

namespace Gdk
{

GC::GC(const Glib::RefPtr<Drawable>& drawable)
: Object( G_OBJECT( gdk_gc_new(drawable->gobj()) ) )
{
  //TODO: We need a construct function if we really want to use our own GClass.
}


}


namespace
{
} // anonymous namespace

// static
GType Glib::Value<Gdk::LineStyle>::value_type()
{
  return gdk_line_style_get_type();
}

// static
GType Glib::Value<Gdk::CapStyle>::value_type()
{
  return gdk_cap_style_get_type();
}

// static
GType Glib::Value<Gdk::JoinStyle>::value_type()
{
  return gdk_join_style_get_type();
}

// static
GType Glib::Value<Gdk::Fill>::value_type()
{
  return gdk_fill_get_type();
}

// static
GType Glib::Value<Gdk::Function>::value_type()
{
  return gdk_function_get_type();
}

// static
GType Glib::Value<Gdk::SubwindowMode>::value_type()
{
  return gdk_subwindow_mode_get_type();
}

// static
GType Glib::Value<Gdk::GCValuesMask>::value_type()
{
  return gdk_gc_values_mask_get_type();
}


namespace Glib
{

Glib::RefPtr<Gdk::GC> wrap(GdkGC* object, bool take_copy /* = false */)
{
  return Glib::RefPtr<Gdk::GC>( dynamic_cast<Gdk::GC*> (Glib::wrap_auto ((GObject*)(object), take_copy)) );
  //We use dynamic_cast<> in case of multiple inheritance.
}

} /* namespace Glib */


namespace Gdk
{


/* The *_Class implementation: */

GType GC_Class::get_type()
  {
    if (!gtype_) //Create the GType if necessary.
      {
        //Make sure that the parent type has been created:
        CppClassParent::CppObjectType::get_type();

        //Create a derived C type.
        //These init funcs will, in turn, call the init functions in the base C type.
        GTypeInfo info =
          {
            sizeof(BaseClassType), /* class_size */
            0, /* base_init */
            0, /* base_finalize */
            (GClassInitFunc) &class_init_function, /* class_init */
            0, /* class_finalize */
            0, /* class_data */
            sizeof(BaseObjectType), /* instance_size */
            0, /* n_preallocs */
            0, /* instance_init */
	    0 /* value_table */
          };

        gtype_ = g_type_register_static(gdk_gc_get_type(), "gdkmm__GC", &info, GTypeFlags(0));

  //Add derived versions of interfaces, if the C type implements any interfaces:
      }
    return gtype_;
  }
void GC_Class::class_init_function(BaseClassType* klass)
{
  CppClassParent::class_init_function((CppClassParent::BaseClassType*) klass);
}


Glib::ObjectBase* GC_Class::wrap_new(GObject* o)
{
  return new GC((GdkGC*)(o));
}


/* The implementation: */

GdkGC* GC::gobj_copy()
{
  reference();
  return gobj();
}

Glib::RefPtr<GC> GC::wrap_specific_type(GdkGC* gobject, bool take_copy /* = false */) //static
{
  Glib::RefPtr<GC> refPtr;

  if(gobject)
  {
    //Check for an existing wrapper:
    GC* pCppObject = dynamic_cast<GC*>(Glib::ObjectBase::_get_current_wrapper(G_OBJECT(gobject)));
    if(pCppObject)
    {
      //Return the existing wrapper:
      refPtr = Glib::RefPtr<GC>(pCppObject);
    }
    else
    {
      //Create a new wrapper:
      refPtr = Glib::RefPtr<GC>( new GC(gobject) );
    }

    if(take_copy && refPtr)
      refPtr->reference();
  }

  return refPtr;
}

GC::GC(GdkGC* castitem)
: Glib::Object((GObject*)(castitem))
{}

GC::~GC()
{
}

GC::CppClassType GC::gc_class_; //Initialize static member.

GType GC::get_type()
{
  return gc_class_.get_type();
}

GType GC::get_base_type()
{
  return gdk_gc_get_type();
}

GC::GC()
: Glib::Object((GObject*)(g_object_new(get_type(), (char*)0)))
{}

Glib::RefPtr<GC> GC::create()
{
  return Glib::RefPtr<GC>( new GC() );
}
Glib::RefPtr<GC> GC::create(const Glib::RefPtr<Drawable>& drawable)
{
  return Glib::RefPtr<GC>( new GC(drawable) );
}
void GC::set_foreground(const Color& color)
{
  gdk_gc_set_foreground(gobj(), const_cast<GdkColor*>(color.gobj()));
}

void GC::set_background(const Color& color)
{
  gdk_gc_set_background(gobj(), const_cast<GdkColor*>(color.gobj()));
}

void GC::set_function(Function function)
{
  gdk_gc_set_function(gobj(), ((GdkFunction)(function)));
}

void GC::set_fill(Fill fill)
{
  gdk_gc_set_fill(gobj(), ((GdkFill)(fill)));
}

void GC::set_tile(const Glib::RefPtr<Pixmap>& tile)
{
  gdk_gc_set_tile(gobj(), Glib::unwrap(tile));
}

void GC::set_stipple(const Glib::RefPtr<Pixmap>& stipple)
{
  gdk_gc_set_stipple(gobj(), Glib::unwrap(stipple));
}

void GC::set_ts_origin(int x, int y)
{
  gdk_gc_set_ts_origin(gobj(), x, y);
}

void GC::set_clip_origin(int x, int y)
{
  gdk_gc_set_clip_origin(gobj(), x, y);
}

void GC::set_clip_mask(const Glib::RefPtr<Bitmap>& mask)
{
  gdk_gc_set_clip_mask(gobj(), Glib::unwrap(mask));
}

void GC::set_clip_rectangle(Rectangle& rectangle)
{
  gdk_gc_set_clip_rectangle(gobj(), (rectangle).gobj());
}

void GC::set_clip_region(GdkRegion* region)
{
  gdk_gc_set_clip_region(gobj(), region);
}

void GC::set_subwindow(SubwindowMode mode)
{
  gdk_gc_set_subwindow(gobj(), ((GdkSubwindowMode)(mode)));
}

void GC::set_exposures(bool exposures)
{
  gdk_gc_set_exposures(gobj(), static_cast<int>(exposures));
}

void GC::set_line_attributes(int line_width, LineStyle line_style, CapStyle cap_style, JoinStyle join_style)
{
  gdk_gc_set_line_attributes(gobj(), line_width, ((GdkLineStyle)(line_style)), ((GdkCapStyle)(cap_style)), ((GdkJoinStyle)(join_style)));
}

void GC::set_dashes(int dash_offset, gint8* dash_list, int n)
{
  gdk_gc_set_dashes(gobj(), dash_offset, dash_list, n);
}

void GC::offset(int x_offset, int y_offset)
{
  gdk_gc_offset(gobj(), x_offset, y_offset);
}

void GC::set_colormap(const Glib::RefPtr<Colormap>& colormap)
{
  gdk_gc_set_colormap(gobj(), Glib::unwrap(colormap));
}

Glib::RefPtr<Colormap> GC::get_colormap()
{
  Glib::RefPtr<Colormap> retvalue = Glib::wrap(gdk_gc_get_colormap(gobj()));
  if(!(retvalue.is_null()))
    retvalue->reference(); //The function does not do a ref for us.
  return retvalue;
}

void GC::set_rgb_fg_color(const Color& color)
{
  gdk_gc_set_rgb_fg_color(gobj(), const_cast<GdkColor*>(color.gobj()));
}

void GC::set_rgb_bg_color(const Color& color)
{
  gdk_gc_set_rgb_bg_color(gobj(), const_cast<GdkColor*>(color.gobj()));
}


} // namespace Gdk

