
#include <gtkmm/button.h>
#include <gtkmm/private/button_p.h>

// -*- c++ -*-
/* $Id: button.ccg,v 1.13 2002/04/02 13:25:13 daniel Exp $ */

/* 
 *
 * Copyright 1998-1999 The Gtk-- Development Team
 * Copyright 2001      Free Software Foundation
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <gtk/gtklabel.h>
#include <gtk/gtkmisc.h>
#include <gtk/gtkbutton.h>
#include <gtkmm/stock.h>


namespace Gtk
{

Button::Button(const Glib::ustring& label, bool mnemonic /* = false */)
:
  Gtk::Bin((GtkBin*) g_object_new(
      get_type(), "label", label.c_str(), "use_underline", gboolean(mnemonic), (char*)0))
{}

Button::Button(const StockID& id)
:
  Gtk::Bin((GtkBin*) g_object_new(
      get_type(), "use_stock", 1, "label", id.get_string().c_str(), (char*)0))
{}

Button::Button(const Stock::BuiltinStockID& id)
:
  Gtk::Bin((GtkBin*) g_object_new(
      get_type(), "use_stock", 1, "label", id.id, (char*)0))
{}

} // namespace Gtk


namespace
{

const Glib::SignalProxyInfo Button_signal_pressed_info =
{
  "pressed",
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback,
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback
};


const Glib::SignalProxyInfo Button_signal_released_info =
{
  "released",
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback,
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback
};


const Glib::SignalProxyInfo Button_signal_clicked_info =
{
  "clicked",
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback,
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback
};


const Glib::SignalProxyInfo Button_signal_enter_info =
{
  "enter",
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback,
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback
};


const Glib::SignalProxyInfo Button_signal_leave_info =
{
  "leave",
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback,
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback
};


const Glib::SignalProxyInfo Button_signal_activate_info =
{
  "activate",
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback,
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback
};

} // anonymous namespace


namespace Glib
{

Gtk::Button* wrap(GtkButton* object, bool take_copy /* = false */)
{
  return dynamic_cast<Gtk::Button *> (Glib::wrap_auto ((GObject*)(object), take_copy));
}

} /* namespace Glib */

namespace Gtk
{


/* The *_Class implementation: */

GType Button_Class::get_type()
  {
    if (!gtype_) //Create the GType if necessary.
      {
        //Make sure that the parent type has been created:
        CppClassParent::CppObjectType::get_type();

        //Create a derived C type.
        //These init funcs will, in turn, call the init functions in the base C type.
        GTypeInfo info =
          {
            sizeof(BaseClassType), /* class_size */
            0, /* base_init */
            0, /* base_finalize */
            (GClassInitFunc) &class_init_function, /* class_init */
            0, /* class_finalize */
            0, /* class_data */
            sizeof(BaseObjectType), /* instance_size */
            0, /* n_preallocs */
            0, /* instance_init */
	    0 /* value_table */
          };

        gtype_ = g_type_register_static(gtk_button_get_type(), "gtkmm__Button", &info, GTypeFlags(0));

  //Add derived versions of interfaces, if the C type implements any interfaces:
      }
    return gtype_;
  }
void Button_Class::class_init_function(BaseClassType* klass)
{
  CppClassParent::class_init_function((CppClassParent::BaseClassType*) klass);
  klass->pressed = &pressed_callback;
  klass->released = &released_callback;
  klass->clicked = &clicked_callback;
  klass->enter = &enter_callback;
  klass->leave = &leave_callback;
  klass->activate = &activate_callback;
}


void Button_Class::pressed_callback(GtkButton* self)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_pressed();
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->pressed)
      (*base->pressed)(self);
  }
}

void Button_Class::released_callback(GtkButton* self)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_released();
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->released)
      (*base->released)(self);
  }
}

void Button_Class::clicked_callback(GtkButton* self)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_clicked();
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->clicked)
      (*base->clicked)(self);
  }
}

void Button_Class::enter_callback(GtkButton* self)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_enter();
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->enter)
      (*base->enter)(self);
  }
}

void Button_Class::leave_callback(GtkButton* self)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_leave();
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->leave)
      (*base->leave)(self);
  }
}

void Button_Class::activate_callback(GtkButton* self)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_activate();
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->activate)
      (*base->activate)(self);
  }
}


Glib::ObjectBase* Button_Class::wrap_new(GObject* o)
{
  return manage(new Button((GtkButton*)(o)));
}


/* The implementation: */

Button::Button(GtkButton* castitem)
: Gtk::Bin((GtkBin*)(castitem))
{}

Button::~Button()
{
  destroy_();
}

Button::CppClassType Button::button_class_; //Initialize static member.

GType Button::get_type()
{
  return button_class_.get_type();
}

GType Button::get_base_type()
{
  return gtk_button_get_type();
}

Button::Button()
: Gtk::Bin((GtkBin*)(g_object_new(get_type(), (char*)0)))
{}

void Button::pressed()
{
  gtk_button_pressed(gobj());
}

void Button::released()
{
  gtk_button_released(gobj());
}

void Button::clicked()
{
  gtk_button_clicked(gobj());
}

void Button::enter()
{
  gtk_button_enter(gobj());
}

void Button::leave()
{
  gtk_button_leave(gobj());
}

void Button::set_relief(ReliefStyle newstyle)
{
  gtk_button_set_relief(gobj(), ((GtkReliefStyle)(newstyle)));
}

ReliefStyle Button::get_relief() const
{
  return ((ReliefStyle)(gtk_button_get_relief(const_cast<GtkButton*>(gobj()))));
}

void Button::set_label(const Glib::ustring& label)
{
  gtk_button_set_label(gobj(), label.c_str());
}

Glib::ustring Button::get_label() const
{
  return Glib::convert_const_gchar_ptr_to_ustring(gtk_button_get_label(const_cast<GtkButton*>(gobj())));
}

void Button::set_use_underline(bool use_underline)
{
  gtk_button_set_use_underline(gobj(), static_cast<int>(use_underline));
}

bool Button::get_use_underline() const
{
  return gtk_button_get_use_underline(const_cast<GtkButton*>(gobj()));
}

void Button::set_use_stock(bool use_stock)
{
  gtk_button_set_use_stock(gobj(), static_cast<int>(use_stock));
}

bool Button::get_use_stock() const
{
  return gtk_button_get_use_stock(const_cast<GtkButton*>(gobj()));
}


Glib::SignalProxy0<void> Button::signal_pressed()
{
  return Glib::SignalProxy0<void>(this, &Button_signal_pressed_info);
}

Glib::SignalProxy0<void> Button::signal_released()
{
  return Glib::SignalProxy0<void>(this, &Button_signal_released_info);
}

Glib::SignalProxy0<void> Button::signal_clicked()
{
  return Glib::SignalProxy0<void>(this, &Button_signal_clicked_info);
}

Glib::SignalProxy0<void> Button::signal_enter()
{
  return Glib::SignalProxy0<void>(this, &Button_signal_enter_info);
}

Glib::SignalProxy0<void> Button::signal_leave()
{
  return Glib::SignalProxy0<void>(this, &Button_signal_leave_info);
}

Glib::SignalProxy0<void> Button::signal_activate()
{
  return Glib::SignalProxy0<void>(this, &Button_signal_activate_info);
}


Glib::PropertyProxy<Glib::ustring> Button::property_label()
{
  return Glib::PropertyProxy<Glib::ustring>(this, "label");
}

Glib::PropertyProxy<ReliefStyle> Button::property_relief()
{
  return Glib::PropertyProxy<ReliefStyle>(this, "relief");
}

Glib::PropertyProxy<bool> Button::property_use_underline()
{
  return Glib::PropertyProxy<bool>(this, "use_underline");
}

Glib::PropertyProxy<bool> Button::property_use_stock()
{
  return Glib::PropertyProxy<bool>(this, "use_stock");
}


void Gtk::Button::on_pressed()
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->pressed)
    (*base->pressed)(gobj());
}

void Gtk::Button::on_released()
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->released)
    (*base->released)(gobj());
}

void Gtk::Button::on_clicked()
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->clicked)
    (*base->clicked)(gobj());
}

void Gtk::Button::on_enter()
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->enter)
    (*base->enter)(gobj());
}

void Gtk::Button::on_leave()
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->leave)
    (*base->leave)(gobj());
}

void Gtk::Button::on_activate()
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->activate)
    (*base->activate)(gobj());
}


} // namespace Gtk

