
#include <gtkmm/curve.h>
#include <gtkmm/private/curve_p.h>

// -*- c++ -*-
/* $Id: curve.ccg,v 1.7 2002/01/18 22:33:32 daniel Exp $ */

/* 
 *
 * Copyright 1998-1999 The Gtk-- Development Team
 * Copyright 2001      Free Software Foundation
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <gtk/gtkcurve.h>
#include <gtk/gtkgamma.h>

namespace Gtk
{

Glib::ArrayHandle<float> Curve::get_vector(int veclen) const
{
  float *const pdata = static_cast<float*>(g_malloc(veclen * sizeof(float)));
  gtk_curve_get_vector(const_cast<GtkCurve*>(gobj()), veclen, pdata);

  return Glib::ArrayHandle<float>(pdata, veclen, Glib::OWNERSHIP_SHALLOW);
}

void Curve::set_vector(const Glib::ArrayHandle<float>& array)
{
  gtk_curve_set_vector(gobj(), array.size(), const_cast<float*>(array.data()));
}

} /* namespace Gtk */


namespace
{

const Glib::SignalProxyInfo Curve_signal_curve_type_changed_info =
{
  "curve_type_changed",
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback,
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback
};

} // anonymous namespace


namespace Glib
{

Gtk::Curve* wrap(GtkCurve* object, bool take_copy /* = false */)
{
  return dynamic_cast<Gtk::Curve *> (Glib::wrap_auto ((GObject*)(object), take_copy));
}

} /* namespace Glib */

namespace Gtk
{


/* The *_Class implementation: */

GType Curve_Class::get_type()
  {
    if (!gtype_) //Create the GType if necessary.
      {
        //Make sure that the parent type has been created:
        CppClassParent::CppObjectType::get_type();

        //Create a derived C type.
        //These init funcs will, in turn, call the init functions in the base C type.
        GTypeInfo info =
          {
            sizeof(BaseClassType), /* class_size */
            0, /* base_init */
            0, /* base_finalize */
            (GClassInitFunc) &class_init_function, /* class_init */
            0, /* class_finalize */
            0, /* class_data */
            sizeof(BaseObjectType), /* instance_size */
            0, /* n_preallocs */
            0, /* instance_init */
	    0 /* value_table */
          };

        gtype_ = g_type_register_static(gtk_curve_get_type(), "gtkmm__Curve", &info, GTypeFlags(0));

  //Add derived versions of interfaces, if the C type implements any interfaces:
      }
    return gtype_;
  }
void Curve_Class::class_init_function(BaseClassType* klass)
{
  CppClassParent::class_init_function((CppClassParent::BaseClassType*) klass);
  klass->curve_type_changed = &curve_type_changed_callback;
}


void Curve_Class::curve_type_changed_callback(GtkCurve* self)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_curve_type_changed();
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->curve_type_changed)
      (*base->curve_type_changed)(self);
  }
}


Glib::ObjectBase* Curve_Class::wrap_new(GObject* o)
{
  return manage(new Curve((GtkCurve*)(o)));
}


/* The implementation: */

Curve::Curve(GtkCurve* castitem)
: Gtk::DrawingArea((GtkDrawingArea*)(castitem))
{}

Curve::~Curve()
{
  destroy_();
}

Curve::CppClassType Curve::curve_class_; //Initialize static member.

GType Curve::get_type()
{
  return curve_class_.get_type();
}

GType Curve::get_base_type()
{
  return gtk_curve_get_type();
}

Curve::Curve()
: Gtk::DrawingArea((GtkDrawingArea*)(g_object_new(get_type(), (char*)0)))
{}

void Curve::reset()
{
  gtk_curve_reset(gobj());
}

void Curve::set_gamma(float gamma)
{
  gtk_curve_set_gamma(gobj(), gamma);
}

void Curve::set_range(float min_x, float max_x, float min_y, float max_y)
{
  gtk_curve_set_range(gobj(), min_x, max_x, min_y, max_y);
}

void Curve::get_vector(int veclen, float* data) const
{
  gtk_curve_get_vector(const_cast<GtkCurve*>(gobj()), veclen, data);
}

void Curve::set_curve_type(CurveType type)
{
  gtk_curve_set_curve_type(gobj(), ((GtkCurveType)(type)));
}


Glib::SignalProxy0<void> Curve::signal_curve_type_changed()
{
  return Glib::SignalProxy0<void>(this, &Curve_signal_curve_type_changed_info);
}


Glib::PropertyProxy<CurveType> Curve::property_curve_type()
{
  return Glib::PropertyProxy<CurveType>(this, "curve_type");
}

Glib::PropertyProxy<float> Curve::property_min_x()
{
  return Glib::PropertyProxy<float>(this, "min_x");
}

Glib::PropertyProxy<float> Curve::property_max_x()
{
  return Glib::PropertyProxy<float>(this, "max_x");
}

Glib::PropertyProxy<float> Curve::property_min_y()
{
  return Glib::PropertyProxy<float>(this, "min_y");
}

Glib::PropertyProxy<float> Curve::property_max_y()
{
  return Glib::PropertyProxy<float>(this, "max_y");
}


void Gtk::Curve::on_curve_type_changed()
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->curve_type_changed)
    (*base->curve_type_changed)(gobj());
}


} // namespace Gtk


namespace Glib
{

Gtk::GammaCurve* wrap(GtkGammaCurve* object, bool take_copy /* = false */)
{
  return dynamic_cast<Gtk::GammaCurve *> (Glib::wrap_auto ((GObject*)(object), take_copy));
}

} /* namespace Glib */

namespace Gtk
{


/* The *_Class implementation: */

GType GammaCurve_Class::get_type()
  {
    if (!gtype_) //Create the GType if necessary.
      {
        //Make sure that the parent type has been created:
        CppClassParent::CppObjectType::get_type();

        //Create a derived C type.
        //These init funcs will, in turn, call the init functions in the base C type.
        GTypeInfo info =
          {
            sizeof(BaseClassType), /* class_size */
            0, /* base_init */
            0, /* base_finalize */
            (GClassInitFunc) &class_init_function, /* class_init */
            0, /* class_finalize */
            0, /* class_data */
            sizeof(BaseObjectType), /* instance_size */
            0, /* n_preallocs */
            0, /* instance_init */
	    0 /* value_table */
          };

        gtype_ = g_type_register_static(gtk_gamma_curve_get_type(), "gtkmm__GammaCurve", &info, GTypeFlags(0));

  //Add derived versions of interfaces, if the C type implements any interfaces:
      }
    return gtype_;
  }
void GammaCurve_Class::class_init_function(BaseClassType* klass)
{
  CppClassParent::class_init_function((CppClassParent::BaseClassType*) klass);
}


Glib::ObjectBase* GammaCurve_Class::wrap_new(GObject* o)
{
  return manage(new GammaCurve((GtkGammaCurve*)(o)));
}


/* The implementation: */

GammaCurve::GammaCurve(GtkGammaCurve* castitem)
: Gtk::VBox((GtkVBox*)(castitem))
{}

GammaCurve::~GammaCurve()
{
  destroy_();
}

GammaCurve::CppClassType GammaCurve::gammacurve_class_; //Initialize static member.

GType GammaCurve::get_type()
{
  return gammacurve_class_.get_type();
}

GType GammaCurve::get_base_type()
{
  return gtk_gamma_curve_get_type();
}

GammaCurve::GammaCurve()
: Gtk::VBox((GtkVBox*)(g_object_new(get_type(), (char*)0)))
{}


} // namespace Gtk

