
#include <gtkmm/textbuffer.h>
#include <gtkmm/private/textbuffer_p.h>

// -*- c++ -*-
/* $Id: textbuffer.ccg,v 1.17 2002/03/26 18:43:28 daniel Exp $ */

/* 
 *
 * Copyright 1998-1999 The Gtk-- Development Team
 * Copyright 2001      Free Software Foundation
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <glib/gslist.h>
#include <gtkmm/textmark.h>

namespace Gtk
{

TextBuffer::TextBuffer()
: Glib::Object((GObject*) g_object_new(get_type(), (char*)0))
{
  gobj()->tag_table = 0; //Code copied from gtk+ function: See bugzilla.gnome.org bug #70227
}

TextBuffer::TextBuffer(const Glib::RefPtr<TextTagTable>& tag_table)
: Glib::Object((GObject*) g_object_new(get_type(), (char*)0))
{
  // FIXME: We need to poke around in the internals, because GtkTextBuffer
  // lacks a "tag_table" property.  This won't be fixed before GTK+ 2.2.

  //Code copied from gtk+ function: See bugzilla.gnome.org bug #70227

  GtkTextTagTable* table_gobj = 0;

  //if(tag_table) //Older compilers seem to have problems with this. But surely we use this technique in lots of places. murrayc
  if(!tag_table.is_null())

  {
    table_gobj = tag_table->gobj_copy();
    table_gobj->buffers = g_slist_prepend(table_gobj->buffers, gobj());
  }

  gobj()->tag_table = table_gobj;
}

Glib::RefPtr<TextTag> TextBuffer::create_tag(const Glib::ustring& tag_name)
{
  //gtk_text_buffer_create_tag takes a varargs list of property names and values.
  //gtkmm coders should use the TextTag.set_* method instead.
  return Glib::wrap(gtk_text_buffer_create_tag(gobj(), tag_name.c_str(), (char*)0), true); //true = take_copy.
  //We have to take a copy because gtk_text_buffer_create_tag() doesn't ref for us, for no real reason.
}

TextIter TextBuffer::get_iter_at_line_offset(int line_number, int char_offset)
{
  TextIter iter;
  gtk_text_buffer_get_iter_at_line_offset(gobj(), iter.gobj(), line_number, char_offset);
  return iter;
}

TextIter TextBuffer::get_iter_at_line_index(int line_number, int byte_index)
{
  TextIter iter;
  gtk_text_buffer_get_iter_at_line_index(gobj(), iter.gobj(), line_number, byte_index);
  return iter;
}

TextIter TextBuffer::get_iter_at_offset(int char_offset)
{
  TextIter iter;
  gtk_text_buffer_get_iter_at_offset(gobj(), iter.gobj(), char_offset);
  return iter;
}

TextIter TextBuffer::get_iter_at_line(int line_number)
{
  TextIter iter;
  gtk_text_buffer_get_iter_at_line(gobj(), iter.gobj(), line_number);
  return iter;
}

TextIter TextBuffer::get_start_iter()
{
  TextIter iter;
  gtk_text_buffer_get_start_iter(gobj(), iter.gobj());
  return iter;
}

TextIter TextBuffer::get_end_iter()
{
  TextIter iter;
  gtk_text_buffer_get_end_iter(gobj(), iter.gobj());
  return iter;
}

void TextBuffer::get_bounds(TextIter& start, TextIter& end)
{
  gtk_text_buffer_get_bounds(gobj(), start.gobj(), end.gobj());
}

TextIter TextBuffer::get_iter_at_mark(const Glib::RefPtr<TextMark>& mark)
{
  TextIter iter;
  gtk_text_buffer_get_iter_at_mark(gobj(), iter.gobj(), mark->gobj());
  return iter;
}

void TextBuffer::get_selection_bounds(TextIter& start, TextIter& end)
{
  gtk_text_buffer_get_selection_bounds(gobj(), start.gobj(), end.gobj());
}

void TextBuffer::set_text(const Glib::ustring& text)
{
  gtk_text_buffer_set_text(gobj(), text.data(), text.bytes());
}

void TextBuffer::set_text(const char* text_begin, const char* text_end)
{
  gtk_text_buffer_set_text(gobj(), text_begin, text_end - text_begin);
}

void TextBuffer::insert(TextIter& iter, const Glib::ustring& text)
{
  gtk_text_buffer_insert(gobj(), iter.gobj(), text.data(), text.bytes());
}

void TextBuffer::insert(TextIter& iter, const char* text_begin, const char* text_end)
{
  gtk_text_buffer_insert(gobj(), iter.gobj(), text_begin, text_end - text_begin);
}

void TextBuffer::insert_at_cursor(const Glib::ustring& text)
{
  gtk_text_buffer_insert_at_cursor(gobj(), text.data(), text.bytes());
}

void TextBuffer::insert_at_cursor(const char* text_begin, const char* text_end)
{
  gtk_text_buffer_insert_at_cursor(gobj(), text_begin, text_end - text_begin);
}

bool TextBuffer::insert_interactive(TextIter& iter, const Glib::ustring& text, bool default_editable)
{
  return gtk_text_buffer_insert_interactive(gobj(), iter.gobj(), text.data(), text.bytes(), default_editable);
}

bool TextBuffer::insert_interactive(TextIter& iter, const char* text_begin, const char* text_end, bool default_editable)
{
  return gtk_text_buffer_insert_interactive(gobj(), iter.gobj(), text_begin, text_end - text_begin, default_editable);
}

bool TextBuffer::insert_interactive_at_cursor(const Glib::ustring& text, bool default_editable)
{
  return gtk_text_buffer_insert_interactive_at_cursor(gobj(), text.data(), text.bytes(), default_editable);
}

bool TextBuffer::insert_interactive_at_cursor(const char* text_begin, const char* text_end, bool default_editable)
{
  return gtk_text_buffer_insert_interactive_at_cursor(gobj(), text_begin, text_end - text_begin, default_editable);
}

void TextBuffer::insert_with_tag(TextIter& iter, const Glib::ustring& text, const Glib::RefPtr<TextTag>& tag)
{
  gtk_text_buffer_insert_with_tags(gobj(), iter.gobj(), text.data(), text.bytes(), tag->gobj(), (GtkTextTag*)0);
}

void TextBuffer::insert_with_tag(TextIter& iter, const char* text_begin, const char* text_end,
                                 const Glib::RefPtr<TextTag>& tag)
{
  gtk_text_buffer_insert_with_tags(gobj(), iter.gobj(), text_begin, text_end - text_begin, tag->gobj(), (GtkTextTag*)0);
}

void TextBuffer::insert_with_tag(TextIter& iter, const Glib::ustring& text, const Glib::ustring& tag_name)
{
  gtk_text_buffer_insert_with_tags_by_name(gobj(), iter.gobj(), text.data(), text.bytes(), tag_name.c_str(), (char*)0);
}

void TextBuffer::insert_with_tag(TextIter& iter, const char* text_begin, const char* text_end,
                                 const Glib::ustring& tag_name)
{
  gtk_text_buffer_insert_with_tags_by_name(
      gobj(), iter.gobj(), text_begin, text_end - text_begin, tag_name.c_str(), (char*)0);
}

void TextBuffer::insert_with_tags(TextIter& iter, const Glib::ustring& text,
                                  const Glib::ArrayHandle< Glib::RefPtr<TextTag> >& tags)
{
  const char *const text_begin = text.data();
  insert_with_tags(iter, text_begin, text_begin + text.bytes(), tags);
}

void TextBuffer::insert_with_tags(TextIter& iter, const char* text_begin, const char* text_end,
                                  const Glib::ArrayHandle< Glib::RefPtr<TextTag> >& tags)
{
  const int start_offset = iter.get_offset();
  insert(iter, text_begin, text_end);

  GtkTextIter start;
  gtk_text_buffer_get_iter_at_offset(gobj(), &start, start_offset);

  GtkTextTag *const *const tags_begin = tags.data();
  GtkTextTag *const *const tags_end   = tags_begin + tags.size();

  for(GtkTextTag *const * ptag = tags_begin; ptag != tags_end; ++ptag)
  {
    gtk_text_buffer_apply_tag(gobj(), *ptag, &start, iter.gobj());
  }
}

void TextBuffer::insert_with_tags_by_name(TextIter& iter, const Glib::ustring& text,
                                          const Glib::ArrayHandle<Glib::ustring>& tag_names)
{
  const char *const text_begin = text.data();
  insert_with_tags_by_name(iter, text_begin, text_begin + text.bytes(), tag_names);
}

void TextBuffer::insert_with_tags_by_name(TextIter& iter, const char* text_begin, const char* text_end,
                                          const Glib::ArrayHandle<Glib::ustring>& tag_names)
{
  const int start_offset = iter.get_offset();
  insert(iter, text_begin, text_end);

  GtkTextIter start;
  gtk_text_buffer_get_iter_at_offset(gobj(), &start, start_offset);

  GtkTextTagTable *const tag_table = gtk_text_buffer_get_tag_table(gobj());

  const char *const *const names_begin = tag_names.data();
  const char *const *const names_end   = names_begin + tag_names.size();

  for(const char *const * pname = names_begin; pname != names_end; ++pname)
  {
    if(GtkTextTag *const tag = gtk_text_tag_table_lookup(tag_table, *pname))
    {
      gtk_text_buffer_apply_tag(gobj(), tag, &start, iter.gobj());
    }
    else
    {
      g_warning("Gtk::TextBuffer::insert_with_tags_by_name(): no tag with name '%s'!", *pname);
      return;
    }
  }
}

TextIter TextBuffer::get_iter_at_child_anchor(const Glib::RefPtr<TextChildAnchor>& anchor)
{
  TextIter iter;
  gtk_text_buffer_get_iter_at_child_anchor(gobj(), iter.gobj(), anchor->gobj());
  return iter;
}

} /* namespace Gtk */


namespace
{


void TextBuffer_signal_insert_text_callback(GtkTextBuffer*, GtkTextIter* p0,const gchar* p1,gint p2,void* data)
{
  using namespace Gtk;
  typedef SigC::Slot3<void,const TextIter&,const Glib::ustring&,int> SlotType;

  try
  {
    SigC::SlotNode *const slot = Glib::SignalProxyNormal::data_to_slot(data);
    (*(SlotType::Proxy)(slot->proxy_))
        (Glib::wrap(p0)
,Glib::convert_const_gchar_ptr_to_ustring(p1)
,p2
, slot);
  }
  catch(...)
  {
    Glib::exception_handlers_invoke();
  }
}

const Glib::SignalProxyInfo TextBuffer_signal_insert_text_info =
{
  "insert_text",
  (GCallback) &TextBuffer_signal_insert_text_callback,
  (GCallback) &TextBuffer_signal_insert_text_callback
};


void TextBuffer_signal_insert_pixbuf_callback(GtkTextBuffer*, GtkTextIter* p0,GdkPixbuf* p1,void* data)
{
  using namespace Gtk;
  typedef SigC::Slot2<void,const TextIter&,const Glib::RefPtr<Gdk::Pixbuf>&> SlotType;

  try
  {
    SigC::SlotNode *const slot = Glib::SignalProxyNormal::data_to_slot(data);
    (*(SlotType::Proxy)(slot->proxy_))
        (Glib::wrap(p0)
,Glib::wrap(p1, true)
, slot);
  }
  catch(...)
  {
    Glib::exception_handlers_invoke();
  }
}

const Glib::SignalProxyInfo TextBuffer_signal_insert_pixbuf_info =
{
  "insert_pixbuf",
  (GCallback) &TextBuffer_signal_insert_pixbuf_callback,
  (GCallback) &TextBuffer_signal_insert_pixbuf_callback
};


void TextBuffer_signal_insert_child_anchor_callback(GtkTextBuffer*, GtkTextIter* p0,GtkTextChildAnchor* p1,void* data)
{
  using namespace Gtk;
  typedef SigC::Slot2<void,const TextIter&,const Glib::RefPtr<TextChildAnchor>&> SlotType;

  try
  {
    SigC::SlotNode *const slot = Glib::SignalProxyNormal::data_to_slot(data);
    (*(SlotType::Proxy)(slot->proxy_))
        (Glib::wrap(p0)
,Glib::wrap(p1, true)
, slot);
  }
  catch(...)
  {
    Glib::exception_handlers_invoke();
  }
}

const Glib::SignalProxyInfo TextBuffer_signal_insert_child_anchor_info =
{
  "insert_child_anchor",
  (GCallback) &TextBuffer_signal_insert_child_anchor_callback,
  (GCallback) &TextBuffer_signal_insert_child_anchor_callback
};


void TextBuffer_signal_delete_range_callback(GtkTextBuffer*, GtkTextIter* p0,GtkTextIter* p1,void* data)
{
  using namespace Gtk;
  typedef SigC::Slot2<void,const TextIter&,const TextIter&> SlotType;

  try
  {
    SigC::SlotNode *const slot = Glib::SignalProxyNormal::data_to_slot(data);
    (*(SlotType::Proxy)(slot->proxy_))
        (Glib::wrap(p0)
,Glib::wrap(p1)
, slot);
  }
  catch(...)
  {
    Glib::exception_handlers_invoke();
  }
}

const Glib::SignalProxyInfo TextBuffer_signal_delete_range_info =
{
  "delete_range",
  (GCallback) &TextBuffer_signal_delete_range_callback,
  (GCallback) &TextBuffer_signal_delete_range_callback
};


const Glib::SignalProxyInfo TextBuffer_signal_changed_info =
{
  "changed",
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback,
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback
};


const Glib::SignalProxyInfo TextBuffer_signal_modified_changed_info =
{
  "modified_changed",
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback,
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback
};


void TextBuffer_signal_mark_set_callback(GtkTextBuffer*, const GtkTextIter* p0,GtkTextMark* p1,void* data)
{
  using namespace Gtk;
  typedef SigC::Slot2<void,const TextIter&,const Glib::RefPtr<TextMark>&> SlotType;

  try
  {
    SigC::SlotNode *const slot = Glib::SignalProxyNormal::data_to_slot(data);
    (*(SlotType::Proxy)(slot->proxy_))
        (Glib::wrap(p0)
,Glib::wrap(p1, true)
, slot);
  }
  catch(...)
  {
    Glib::exception_handlers_invoke();
  }
}

const Glib::SignalProxyInfo TextBuffer_signal_mark_set_info =
{
  "mark_set",
  (GCallback) &TextBuffer_signal_mark_set_callback,
  (GCallback) &TextBuffer_signal_mark_set_callback
};


void TextBuffer_signal_mark_deleted_callback(GtkTextBuffer*, GtkTextMark* p0,void* data)
{
  using namespace Gtk;
  typedef SigC::Slot1<void,const Glib::RefPtr<TextMark>&> SlotType;

  try
  {
    SigC::SlotNode *const slot = Glib::SignalProxyNormal::data_to_slot(data);
    (*(SlotType::Proxy)(slot->proxy_))
        (Glib::wrap(p0, true)
, slot);
  }
  catch(...)
  {
    Glib::exception_handlers_invoke();
  }
}

const Glib::SignalProxyInfo TextBuffer_signal_mark_deleted_info =
{
  "mark_deleted",
  (GCallback) &TextBuffer_signal_mark_deleted_callback,
  (GCallback) &TextBuffer_signal_mark_deleted_callback
};


void TextBuffer_signal_apply_tag_callback(GtkTextBuffer*, GtkTextTag* p0,const GtkTextIter* p1,const GtkTextIter* p2,void* data)
{
  using namespace Gtk;
  typedef SigC::Slot3<void,const Glib::RefPtr<TextTag>&,const TextIter&,const TextIter&> SlotType;

  try
  {
    SigC::SlotNode *const slot = Glib::SignalProxyNormal::data_to_slot(data);
    (*(SlotType::Proxy)(slot->proxy_))
        (Glib::wrap(p0, true)
,Glib::wrap(p1)
,Glib::wrap(p2)
, slot);
  }
  catch(...)
  {
    Glib::exception_handlers_invoke();
  }
}

const Glib::SignalProxyInfo TextBuffer_signal_apply_tag_info =
{
  "apply_tag",
  (GCallback) &TextBuffer_signal_apply_tag_callback,
  (GCallback) &TextBuffer_signal_apply_tag_callback
};


void TextBuffer_signal_remove_tag_callback(GtkTextBuffer*, GtkTextTag* p0,const GtkTextIter* p1,const GtkTextIter* p2,void* data)
{
  using namespace Gtk;
  typedef SigC::Slot3<void,const Glib::RefPtr<TextTag>&,const TextIter&,const TextIter&> SlotType;

  try
  {
    SigC::SlotNode *const slot = Glib::SignalProxyNormal::data_to_slot(data);
    (*(SlotType::Proxy)(slot->proxy_))
        (Glib::wrap(p0, true)
,Glib::wrap(p1)
,Glib::wrap(p2)
, slot);
  }
  catch(...)
  {
    Glib::exception_handlers_invoke();
  }
}

const Glib::SignalProxyInfo TextBuffer_signal_remove_tag_info =
{
  "remove_tag",
  (GCallback) &TextBuffer_signal_remove_tag_callback,
  (GCallback) &TextBuffer_signal_remove_tag_callback
};


const Glib::SignalProxyInfo TextBuffer_signal_begin_user_action_info =
{
  "begin_user_action",
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback,
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback
};


const Glib::SignalProxyInfo TextBuffer_signal_end_user_action_info =
{
  "end_user_action",
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback,
  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback
};

} // anonymous namespace


namespace Glib
{

Glib::RefPtr<Gtk::TextBuffer> wrap(GtkTextBuffer* object, bool take_copy /* = false */)
{
  return Glib::RefPtr<Gtk::TextBuffer>( dynamic_cast<Gtk::TextBuffer*> (Glib::wrap_auto ((GObject*)(object), take_copy)) );
  //We use dynamic_cast<> in case of multiple inheritance.
}

} /* namespace Glib */


namespace Gtk
{


/* The *_Class implementation: */

GType TextBuffer_Class::get_type()
  {
    if (!gtype_) //Create the GType if necessary.
      {
        //Make sure that the parent type has been created:
        CppClassParent::CppObjectType::get_type();

        //Create a derived C type.
        //These init funcs will, in turn, call the init functions in the base C type.
        GTypeInfo info =
          {
            sizeof(BaseClassType), /* class_size */
            0, /* base_init */
            0, /* base_finalize */
            (GClassInitFunc) &class_init_function, /* class_init */
            0, /* class_finalize */
            0, /* class_data */
            sizeof(BaseObjectType), /* instance_size */
            0, /* n_preallocs */
            0, /* instance_init */
	    0 /* value_table */
          };

        gtype_ = g_type_register_static(gtk_text_buffer_get_type(), "gtkmm__TextBuffer", &info, GTypeFlags(0));

  //Add derived versions of interfaces, if the C type implements any interfaces:
      }
    return gtype_;
  }
void TextBuffer_Class::class_init_function(BaseClassType* klass)
{
  CppClassParent::class_init_function((CppClassParent::BaseClassType*) klass);
  klass->insert_text = &insert_text_callback;
  klass->insert_pixbuf = &insert_pixbuf_callback;
  klass->insert_child_anchor = &insert_child_anchor_callback;
  klass->delete_range = &delete_range_callback;
  klass->changed = &changed_callback;
  klass->modified_changed = &modified_changed_callback;
  klass->mark_set = &mark_set_callback;
  klass->mark_deleted = &mark_deleted_callback;
  klass->apply_tag = &apply_tag_callback;
  klass->remove_tag = &remove_tag_callback;
  klass->begin_user_action = &begin_user_action_callback;
  klass->end_user_action = &end_user_action_callback;
}


void TextBuffer_Class::insert_text_callback(GtkTextBuffer* self, GtkTextIter* p0, const gchar* p1, gint p2)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_insert_text(Glib::wrap(p0)
, Glib::convert_const_gchar_ptr_to_ustring(p1)
, p2
);
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->insert_text)
      (*base->insert_text)(self, p0, p1, p2);
  }
}

void TextBuffer_Class::insert_pixbuf_callback(GtkTextBuffer* self, GtkTextIter* p0, GdkPixbuf* p1)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_insert_pixbuf(Glib::wrap(p0)
, Glib::wrap(p1, true)
);
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->insert_pixbuf)
      (*base->insert_pixbuf)(self, p0, p1);
  }
}

void TextBuffer_Class::insert_child_anchor_callback(GtkTextBuffer* self, GtkTextIter* p0, GtkTextChildAnchor* p1)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_insert_child_anchor(Glib::wrap(p0)
, Glib::wrap(p1, true)
);
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->insert_child_anchor)
      (*base->insert_child_anchor)(self, p0, p1);
  }
}

void TextBuffer_Class::delete_range_callback(GtkTextBuffer* self, GtkTextIter* p0, GtkTextIter* p1)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_delete_range(Glib::wrap(p0)
, Glib::wrap(p1)
);
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->delete_range)
      (*base->delete_range)(self, p0, p1);
  }
}

void TextBuffer_Class::changed_callback(GtkTextBuffer* self)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_changed();
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->changed)
      (*base->changed)(self);
  }
}

void TextBuffer_Class::modified_changed_callback(GtkTextBuffer* self)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_modified_changed();
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->modified_changed)
      (*base->modified_changed)(self);
  }
}

void TextBuffer_Class::mark_set_callback(GtkTextBuffer* self, const GtkTextIter* p0, GtkTextMark* p1)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_mark_set(Glib::wrap(p0)
, Glib::wrap(p1, true)
);
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->mark_set)
      (*base->mark_set)(self, p0, p1);
  }
}

void TextBuffer_Class::mark_deleted_callback(GtkTextBuffer* self, GtkTextMark* p0)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_mark_deleted(Glib::wrap(p0, true)
);
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->mark_deleted)
      (*base->mark_deleted)(self, p0);
  }
}

void TextBuffer_Class::apply_tag_callback(GtkTextBuffer* self, GtkTextTag* p0, const GtkTextIter* p1, const GtkTextIter* p2)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_apply_tag(Glib::wrap(p0, true)
, Glib::wrap(p1)
, Glib::wrap(p2)
);
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->apply_tag)
      (*base->apply_tag)(self, p0, p1, p2);
  }
}

void TextBuffer_Class::remove_tag_callback(GtkTextBuffer* self, GtkTextTag* p0, const GtkTextIter* p1, const GtkTextIter* p2)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_remove_tag(Glib::wrap(p0, true)
, Glib::wrap(p1)
, Glib::wrap(p2)
);
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->remove_tag)
      (*base->remove_tag)(self, p0, p1, p2);
  }
}

void TextBuffer_Class::begin_user_action_callback(GtkTextBuffer* self)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_begin_user_action();
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->begin_user_action)
      (*base->begin_user_action)(self);
  }
}

void TextBuffer_Class::end_user_action_callback(GtkTextBuffer* self)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_end_user_action();
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->end_user_action)
      (*base->end_user_action)(self);
  }
}


Glib::ObjectBase* TextBuffer_Class::wrap_new(GObject* o)
{
  return new TextBuffer((GtkTextBuffer*)(o));
}


/* The implementation: */

GtkTextBuffer* TextBuffer::gobj_copy()
{
  reference();
  return gobj();
}

Glib::RefPtr<TextBuffer> TextBuffer::wrap_specific_type(GtkTextBuffer* gobject, bool take_copy /* = false */) //static
{
  Glib::RefPtr<TextBuffer> refPtr;

  if(gobject)
  {
    //Check for an existing wrapper:
    TextBuffer* pCppObject = dynamic_cast<TextBuffer*>(Glib::ObjectBase::_get_current_wrapper(G_OBJECT(gobject)));
    if(pCppObject)
    {
      //Return the existing wrapper:
      refPtr = Glib::RefPtr<TextBuffer>(pCppObject);
    }
    else
    {
      //Create a new wrapper:
      refPtr = Glib::RefPtr<TextBuffer>( new TextBuffer(gobject) );
    }

    if(take_copy && refPtr)
      refPtr->reference();
  }

  return refPtr;
}

TextBuffer::TextBuffer(GtkTextBuffer* castitem)
: Glib::Object((GObject*)(castitem))
{}

TextBuffer::~TextBuffer()
{
}

TextBuffer::CppClassType TextBuffer::textbuffer_class_; //Initialize static member.

GType TextBuffer::get_type()
{
  return textbuffer_class_.get_type();
}

GType TextBuffer::get_base_type()
{
  return gtk_text_buffer_get_type();
}

Glib::RefPtr<TextBuffer> TextBuffer::create()
{
  return Glib::RefPtr<TextBuffer>( new TextBuffer() );
}
Glib::RefPtr<TextBuffer> TextBuffer::create(const Glib::RefPtr<TextTagTable>& tag_table)
{
  return Glib::RefPtr<TextBuffer>( new TextBuffer(tag_table) );
}
int TextBuffer::get_line_count() const
{
  return gtk_text_buffer_get_line_count(const_cast<GtkTextBuffer*>(gobj()));
}

int TextBuffer::get_char_count() const
{
  return gtk_text_buffer_get_char_count(const_cast<GtkTextBuffer*>(gobj()));
}

Glib::RefPtr<TextTagTable> TextBuffer::get_tag_table()
{
  Glib::RefPtr<TextTagTable> retvalue = Glib::wrap(gtk_text_buffer_get_tag_table(gobj()));
  if(!(retvalue.is_null()))
    retvalue->reference(); //The function does not do a ref for us.
  return retvalue;
}

void TextBuffer::insert_range(TextIter& iter, const TextIter& start, const TextIter& end)
{
  gtk_text_buffer_insert_range(gobj(), (iter).gobj(), (start).gobj(), (end).gobj());
}

bool TextBuffer::insert_range_interactive(TextIter& iter, const TextIter& start, const TextIter& end, bool default_editable)
{
  return gtk_text_buffer_insert_range_interactive(gobj(), (iter).gobj(), (start).gobj(), (end).gobj(), static_cast<int>(default_editable));
}

void TextBuffer::delete_(TextIter& start, TextIter& end)
{
  gtk_text_buffer_delete(gobj(), (start).gobj(), (end).gobj());
}

bool TextBuffer::delete_interactive(TextIter& start_iter, TextIter& end_iter, bool default_editable)
{
  return gtk_text_buffer_delete_interactive(gobj(), (start_iter).gobj(), (end_iter).gobj(), static_cast<int>(default_editable));
}

Glib::ustring TextBuffer::get_text(const TextIter& start, const TextIter& end, bool include_hidden_chars)
{
  return Glib::convert_return_gchar_ptr_to_ustring(gtk_text_buffer_get_text(gobj(), (start).gobj(), (end).gobj(), static_cast<int>(include_hidden_chars)));
}

Glib::ustring TextBuffer::get_slice(const TextIter& start, const TextIter& end, bool include_hidden_chars)
{
  return Glib::convert_return_gchar_ptr_to_ustring(gtk_text_buffer_get_slice(gobj(), (start).gobj(), (end).gobj(), static_cast<int>(include_hidden_chars)));
}

void TextBuffer::insert_pixbuf(TextIter& iter, const Glib::RefPtr<Gdk::Pixbuf>& pixbuf)
{
  gtk_text_buffer_insert_pixbuf(gobj(), (iter).gobj(), Glib::unwrap(pixbuf));
}

void TextBuffer::insert_child_anchor(TextIter& iter, const Glib::RefPtr<TextChildAnchor>& anchor)
{
  gtk_text_buffer_insert_child_anchor(gobj(), (iter).gobj(), Glib::unwrap(anchor));
}

Glib::RefPtr<TextChildAnchor> TextBuffer::create_child_anchor(TextIter& iter)
{
  Glib::RefPtr<TextChildAnchor> retvalue = Glib::wrap(gtk_text_buffer_create_child_anchor(gobj(), (iter).gobj()));
  if(!(retvalue.is_null()))
    retvalue->reference(); //The function does not do a ref for us.
  return retvalue;
}

Glib::RefPtr<TextMark> TextBuffer::create_mark(const Glib::ustring& mark_name, const TextIter& where, bool left_gravity)
{
  return Glib::wrap(gtk_text_buffer_create_mark(gobj(), mark_name.c_str(), (where).gobj(), static_cast<int>(left_gravity)));
}

void TextBuffer::move_mark(const Glib::RefPtr<TextMark>& mark, const TextIter& where)
{
  gtk_text_buffer_move_mark(gobj(), Glib::unwrap(mark), (where).gobj());
}

void TextBuffer::delete_mark(const Glib::RefPtr<TextMark>& mark)
{
  gtk_text_buffer_delete_mark(gobj(), Glib::unwrap(mark));
}

Glib::RefPtr<TextMark> TextBuffer::get_mark(const Glib::ustring& name)
{
  Glib::RefPtr<TextMark> retvalue = Glib::wrap(gtk_text_buffer_get_mark(gobj(), name.c_str()));
  if(!(retvalue.is_null()))
    retvalue->reference(); //The function does not do a ref for us.
  return retvalue;
}

void TextBuffer::move_mark_by_name(const Glib::ustring& name, const TextIter& where)
{
  gtk_text_buffer_move_mark_by_name(gobj(), name.c_str(), (where).gobj());
}

void TextBuffer::delete_mark_by_name(const Glib::ustring& name)
{
  gtk_text_buffer_delete_mark_by_name(gobj(), name.c_str());
}

Glib::RefPtr<TextMark> TextBuffer::get_insert()
{
  Glib::RefPtr<TextMark> retvalue = Glib::wrap(gtk_text_buffer_get_insert(gobj()));
  if(!(retvalue.is_null()))
    retvalue->reference(); //The function does not do a ref for us.
  return retvalue;
}

Glib::RefPtr<TextMark> TextBuffer::get_selection_bound()
{
  Glib::RefPtr<TextMark> retvalue = Glib::wrap(gtk_text_buffer_get_selection_bound(gobj()));
  if(!(retvalue.is_null()))
    retvalue->reference(); //The function does not do a ref for us.
  return retvalue;
}

void TextBuffer::place_cursor(const TextIter& where)
{
  gtk_text_buffer_place_cursor(gobj(), (where).gobj());
}

void TextBuffer::apply_tag(const Glib::RefPtr<TextTag>& tag, const TextIter& start, const TextIter& end)
{
  gtk_text_buffer_apply_tag(gobj(), Glib::unwrap(tag), (start).gobj(), (end).gobj());
}

void TextBuffer::remove_tag(const Glib::RefPtr<TextTag>& tag, const TextIter& start, const TextIter& end)
{
  gtk_text_buffer_remove_tag(gobj(), Glib::unwrap(tag), (start).gobj(), (end).gobj());
}

void TextBuffer::apply_tag_by_name(const Glib::ustring& name, const TextIter& start, const TextIter& end)
{
  gtk_text_buffer_apply_tag_by_name(gobj(), name.c_str(), (start).gobj(), (end).gobj());
}

void TextBuffer::remove_tag_by_name(const Glib::ustring& name, const TextIter& start, const TextIter& end)
{
  gtk_text_buffer_remove_tag_by_name(gobj(), name.c_str(), (start).gobj(), (end).gobj());
}

void TextBuffer::remove_all_tags(const TextIter& start, const TextIter& end)
{
  gtk_text_buffer_remove_all_tags(gobj(), (start).gobj(), (end).gobj());
}

bool TextBuffer::get_modified()
{
  return gtk_text_buffer_get_modified(gobj());
}

void TextBuffer::set_modified(bool setting)
{
  gtk_text_buffer_set_modified(gobj(), static_cast<int>(setting));
}

bool TextBuffer::delete_selection(bool interactive, bool default_editable)
{
  return gtk_text_buffer_delete_selection(gobj(), static_cast<int>(interactive), static_cast<int>(default_editable));
}

void TextBuffer::begin_user_action()
{
  gtk_text_buffer_begin_user_action(gobj());
}

void TextBuffer::end_user_action()
{
  gtk_text_buffer_end_user_action(gobj());
}


Glib::SignalProxy3<void,const TextIter&,const Glib::ustring&,int> TextBuffer::signal_insert_text()
{
  return Glib::SignalProxy3<void,const TextIter&,const Glib::ustring&,int>(this, &TextBuffer_signal_insert_text_info);
}

Glib::SignalProxy2<void,const TextIter&,const Glib::RefPtr<Gdk::Pixbuf>&> TextBuffer::signal_insert_pixbuf()
{
  return Glib::SignalProxy2<void,const TextIter&,const Glib::RefPtr<Gdk::Pixbuf>&>(this, &TextBuffer_signal_insert_pixbuf_info);
}

Glib::SignalProxy2<void,const TextIter&,const Glib::RefPtr<TextChildAnchor>&> TextBuffer::signal_insert_child_anchor()
{
  return Glib::SignalProxy2<void,const TextIter&,const Glib::RefPtr<TextChildAnchor>&>(this, &TextBuffer_signal_insert_child_anchor_info);
}

Glib::SignalProxy2<void,const TextIter&,const TextIter&> TextBuffer::signal_delete_range()
{
  return Glib::SignalProxy2<void,const TextIter&,const TextIter&>(this, &TextBuffer_signal_delete_range_info);
}

Glib::SignalProxy0<void> TextBuffer::signal_changed()
{
  return Glib::SignalProxy0<void>(this, &TextBuffer_signal_changed_info);
}

Glib::SignalProxy0<void> TextBuffer::signal_modified_changed()
{
  return Glib::SignalProxy0<void>(this, &TextBuffer_signal_modified_changed_info);
}

Glib::SignalProxy2<void,const TextIter&,const Glib::RefPtr<TextMark>&> TextBuffer::signal_mark_set()
{
  return Glib::SignalProxy2<void,const TextIter&,const Glib::RefPtr<TextMark>&>(this, &TextBuffer_signal_mark_set_info);
}

Glib::SignalProxy1<void,const Glib::RefPtr<TextMark>&> TextBuffer::signal_mark_deleted()
{
  return Glib::SignalProxy1<void,const Glib::RefPtr<TextMark>&>(this, &TextBuffer_signal_mark_deleted_info);
}

Glib::SignalProxy3<void,const Glib::RefPtr<TextTag>&,const TextIter&,const TextIter&> TextBuffer::signal_apply_tag()
{
  return Glib::SignalProxy3<void,const Glib::RefPtr<TextTag>&,const TextIter&,const TextIter&>(this, &TextBuffer_signal_apply_tag_info);
}

Glib::SignalProxy3<void,const Glib::RefPtr<TextTag>&,const TextIter&,const TextIter&> TextBuffer::signal_remove_tag()
{
  return Glib::SignalProxy3<void,const Glib::RefPtr<TextTag>&,const TextIter&,const TextIter&>(this, &TextBuffer_signal_remove_tag_info);
}

Glib::SignalProxy0<void> TextBuffer::signal_begin_user_action()
{
  return Glib::SignalProxy0<void>(this, &TextBuffer_signal_begin_user_action_info);
}

Glib::SignalProxy0<void> TextBuffer::signal_end_user_action()
{
  return Glib::SignalProxy0<void>(this, &TextBuffer_signal_end_user_action_info);
}


void Gtk::TextBuffer::on_insert_text(const TextIter& pos, const Glib::ustring& text, int bytes)
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->insert_text)
    (*base->insert_text)(gobj(),const_cast<GtkTextIter*>((pos).gobj()),text.c_str(),bytes);
}

void Gtk::TextBuffer::on_insert_pixbuf(const TextIter& pos, const Glib::RefPtr<Gdk::Pixbuf>& pixbuf)
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->insert_pixbuf)
    (*base->insert_pixbuf)(gobj(),const_cast<GtkTextIter*>((pos).gobj()),Glib::unwrap(pixbuf));
}

void Gtk::TextBuffer::on_insert_child_anchor(const TextIter& pos, const Glib::RefPtr<TextChildAnchor>& anchor)
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->insert_child_anchor)
    (*base->insert_child_anchor)(gobj(),const_cast<GtkTextIter*>((pos).gobj()),Glib::unwrap(anchor));
}

void Gtk::TextBuffer::on_delete_range(const TextIter& start, const TextIter& end)
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->delete_range)
    (*base->delete_range)(gobj(),const_cast<GtkTextIter*>((start).gobj()),const_cast<GtkTextIter*>((end).gobj()));
}

void Gtk::TextBuffer::on_changed()
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->changed)
    (*base->changed)(gobj());
}

void Gtk::TextBuffer::on_modified_changed()
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->modified_changed)
    (*base->modified_changed)(gobj());
}

void Gtk::TextBuffer::on_mark_set(const TextIter& location, const Glib::RefPtr<TextMark>& mark)
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->mark_set)
    (*base->mark_set)(gobj(),(location).gobj(),Glib::unwrap(mark));
}

void Gtk::TextBuffer::on_mark_deleted(const Glib::RefPtr<TextMark>& mark)
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->mark_deleted)
    (*base->mark_deleted)(gobj(),Glib::unwrap(mark));
}

void Gtk::TextBuffer::on_apply_tag(const Glib::RefPtr<TextTag>& tag, const TextIter& start_char, const TextIter& end_char)
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->apply_tag)
    (*base->apply_tag)(gobj(),Glib::unwrap(tag),(start_char).gobj(),(end_char).gobj());
}

void Gtk::TextBuffer::on_remove_tag(const Glib::RefPtr<TextTag>& tag, const TextIter& start_char, const TextIter& end_char)
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->remove_tag)
    (*base->remove_tag)(gobj(),Glib::unwrap(tag),(start_char).gobj(),(end_char).gobj());
}

void Gtk::TextBuffer::on_begin_user_action()
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->begin_user_action)
    (*base->begin_user_action)(gobj());
}

void Gtk::TextBuffer::on_end_user_action()
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->end_user_action)
    (*base->end_user_action)(gobj());
}


} // namespace Gtk

