
#include <gtkmm/treestore.h>
#include <gtkmm/private/treestore_p.h>

// -*- c++ -*-
/* $Id: treestore.ccg,v 1.21 2002/04/08 17:20:02 daniel Exp $ */

/* 
 *
 * Copyright 1998-1999 The Gtk-- Development Team
 * Copyright 2001      Free Software Foundation
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <gtkmm/textmark.h>
#include <glibmm/value.h>

namespace Gtk
{

TreeStore::TreeStore(const TreeModelColumnRecord& columns)
:
  Glib::Object((GObject*) g_object_new(get_type(), (char*)0))
{
  gtk_tree_store_set_column_types(gobj(), columns.size(), const_cast<GType*>(columns.types()));
}

TreeIter TreeStore::erase(const TreeIter& iter)
{
  g_assert(iter.get_gobject_if_not_end() != 0);

  TreeIter next (iter);
  ++next;

  GtkTreeIter tmp = *iter.gobj();
  gtk_tree_store_remove(gobj(), &tmp);

  return next;
}

TreeIter TreeStore::insert(const TreeIter& iter)
{
  TreeIter new_pos (TreeModel::gobj());

  gtk_tree_store_insert_before(
      gobj(), new_pos.gobj(),
      const_cast<GtkTreeIter*>(iter.get_parent_gobject_if_end()),
      const_cast<GtkTreeIter*>(iter.get_gobject_if_not_end()));

  if(new_pos.gobj()->stamp == 0)
    new_pos.setup_end_iterator(iter);

  return new_pos;
}

TreeIter TreeStore::prepend()
{
  TreeIter new_pos (TreeModel::gobj());
  gtk_tree_store_prepend(gobj(), new_pos.gobj(), 0);
  return new_pos;
}

TreeIter TreeStore::prepend(const TreeIterChildren& node)
{
  TreeIter new_pos (TreeModel::gobj());

  gtk_tree_store_prepend(
      gobj(), new_pos.gobj(),
      const_cast<GtkTreeIter*>(node.get_parent_gobject()));

  return new_pos;
}

TreeIter TreeStore::append()
{
  TreeIter new_pos (TreeModel::gobj());
  gtk_tree_store_append(gobj(), new_pos.gobj(), 0);
  return new_pos;
}

TreeIter TreeStore::append(const TreeIterChildren& node)
{
  TreeIter new_pos (TreeModel::gobj());

  gtk_tree_store_append(
      gobj(), new_pos.gobj(),
      const_cast<GtkTreeIter*>(node.get_parent_gobject()));

  return new_pos;
}

} // namespace Gtk


namespace
{
} // anonymous namespace


namespace Glib
{

Glib::RefPtr<Gtk::TreeStore> wrap(GtkTreeStore* object, bool take_copy /* = false */)
{
  return Glib::RefPtr<Gtk::TreeStore>( dynamic_cast<Gtk::TreeStore*> (Glib::wrap_auto ((GObject*)(object), take_copy)) );
  //We use dynamic_cast<> in case of multiple inheritance.
}

} /* namespace Glib */


namespace Gtk
{


/* The *_Class implementation: */

GType TreeStore_Class::get_type()
  {
    if (!gtype_) //Create the GType if necessary.
      {
        //Make sure that the parent type has been created:
        CppClassParent::CppObjectType::get_type();

        //Create a derived C type.
        //These init funcs will, in turn, call the init functions in the base C type.
        GTypeInfo info =
          {
            sizeof(BaseClassType), /* class_size */
            0, /* base_init */
            0, /* base_finalize */
            (GClassInitFunc) &class_init_function, /* class_init */
            0, /* class_finalize */
            0, /* class_data */
            sizeof(BaseObjectType), /* instance_size */
            0, /* n_preallocs */
            0, /* instance_init */
	    0 /* value_table */
          };

        gtype_ = g_type_register_static(gtk_tree_store_get_type(), "gtkmm__TreeStore", &info, GTypeFlags(0));

  //Add derived versions of interfaces, if the C type implements any interfaces:
  TreeModel::add_interface(get_type());
  TreeSortable::add_interface(get_type());
  TreeDragSource::add_interface(get_type());
  TreeDragDest::add_interface(get_type());
      }
    return gtype_;
  }
void TreeStore_Class::class_init_function(BaseClassType* klass)
{
  CppClassParent::class_init_function((CppClassParent::BaseClassType*) klass);
}


Glib::ObjectBase* TreeStore_Class::wrap_new(GObject* o)
{
  return new TreeStore((GtkTreeStore*)(o));
}


/* The implementation: */

GtkTreeStore* TreeStore::gobj_copy()
{
  reference();
  return gobj();
}

Glib::RefPtr<TreeStore> TreeStore::wrap_specific_type(GtkTreeStore* gobject, bool take_copy /* = false */) //static
{
  Glib::RefPtr<TreeStore> refPtr;

  if(gobject)
  {
    //Check for an existing wrapper:
    TreeStore* pCppObject = dynamic_cast<TreeStore*>(Glib::ObjectBase::_get_current_wrapper(G_OBJECT(gobject)));
    if(pCppObject)
    {
      //Return the existing wrapper:
      refPtr = Glib::RefPtr<TreeStore>(pCppObject);
    }
    else
    {
      //Create a new wrapper:
      refPtr = Glib::RefPtr<TreeStore>( new TreeStore(gobject) );
    }

    if(take_copy && refPtr)
      refPtr->reference();
  }

  return refPtr;
}

TreeStore::TreeStore(GtkTreeStore* castitem)
: Glib::Object((GObject*)(castitem))
{}

TreeStore::~TreeStore()
{
}

TreeStore::CppClassType TreeStore::treestore_class_; //Initialize static member.

GType TreeStore::get_type()
{
  return treestore_class_.get_type();
}

GType TreeStore::get_base_type()
{
  return gtk_tree_store_get_type();
}

Glib::RefPtr<TreeStore> TreeStore::create(const TreeModelColumnRecord& columns)
{
  return Glib::RefPtr<TreeStore>( new TreeStore(columns) );
}
void TreeStore::clear()
{
  gtk_tree_store_clear(gobj());
}

bool TreeStore::is_ancestor(const TreeIter& iter, const TreeIter& descendant)
{
  return gtk_tree_store_is_ancestor(gobj(), const_cast<GtkTreeIter*>((iter).gobj()), const_cast<GtkTreeIter*>((descendant).gobj()));
}

int TreeStore::iter_depth(const TreeIter& iter)
{
  return gtk_tree_store_iter_depth(gobj(), const_cast<GtkTreeIter*>((iter).gobj()));
}

void TreeStore::set_value_impl(const TreeIter& row, int column, const Glib::ValueBase& value)
{
  gtk_tree_store_set_value(gobj(), const_cast<GtkTreeIter*>((row).gobj()), column, const_cast<GValue*>(value.gobj()));
}


} // namespace Gtk

