
#include <pangomm/context.h>
#include <pangomm/private/context_p.h>

#include <pango/pango-enum-types.h>
// -*- c++ -*-
/* $Id: context.ccg,v 1.3 2002/02/24 16:14:07 cactus Exp $ */

/* 
 *
 * Copyright 1998-1999 The Gtk-- Development Team
 * Copyright 2001      Free Software Foundation
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

namespace Pango
{

Glib::ArrayHandle< Glib::RefPtr < FontFamily >, FontFamilyRefPtrTraits > Context::list_families()
{
  //Get array:
  PangoFontFamily** pFamilies = 0;
  int n_families = 0;
  pango_context_list_families(gobj(), &pFamilies, &n_families);
  
  return Glib::ArrayHandle< Glib::RefPtr<FontFamily>, FontFamilyRefPtrTraits >
      (pFamilies, n_families, Glib::OWNERSHIP_SHALLOW);
}


} /* namespace Pango */


namespace
{
} // anonymous namespace

// static
GType Glib::Value<Pango::Direction>::value_type()
{
  return pango_direction_get_type();
}


namespace Glib
{

Glib::RefPtr<Pango::Context> wrap(PangoContext* object, bool take_copy /* = false */)
{
  return Glib::RefPtr<Pango::Context>( dynamic_cast<Pango::Context*> (Glib::wrap_auto ((GObject*)(object), take_copy)) );
  //We use dynamic_cast<> in case of multiple inheritance.
}

} /* namespace Glib */


namespace Pango
{


/* The *_Class implementation: */

GType Context_Class::get_type()
  {
    gtype_ = pango_context_get_type(); //The actual struct definition is hidden, so we can not derive one.
    return gtype_;
  }


Glib::ObjectBase* Context_Class::wrap_new(GObject* o)
{
  return new Context((PangoContext*)(o));
}


/* The implementation: */

PangoContext* Context::gobj_copy()
{
  reference();
  return gobj();
}

Glib::RefPtr<Context> Context::wrap_specific_type(PangoContext* gobject, bool take_copy /* = false */) //static
{
  Glib::RefPtr<Context> refPtr;

  if(gobject)
  {
    //Check for an existing wrapper:
    Context* pCppObject = dynamic_cast<Context*>(Glib::ObjectBase::_get_current_wrapper(G_OBJECT(gobject)));
    if(pCppObject)
    {
      //Return the existing wrapper:
      refPtr = Glib::RefPtr<Context>(pCppObject);
    }
    else
    {
      //Create a new wrapper:
      refPtr = Glib::RefPtr<Context>( new Context(gobject) );
    }

    if(take_copy && refPtr)
      refPtr->reference();
  }

  return refPtr;
}

Context::Context(PangoContext* castitem)
: Glib::Object((GObject*)(castitem))
{}

Context::~Context()
{
}

Context::CppClassType Context::context_class_; //Initialize static member.

GType Context::get_type()
{
  return context_class_.get_type();
}

GType Context::get_base_type()
{
  return pango_context_get_type();
}

Context::Context()
: Glib::Object((GObject*)(g_object_new(get_type(), (char*)0)))
{}

Glib::RefPtr<Context> Context::create()
{
  return Glib::RefPtr<Context>( new Context() );
}
Glib::RefPtr<Font> Context::load_font(const FontDescription& desc)
{
  return Glib::wrap(pango_context_load_font(gobj(), (desc).gobj()));
}

Glib::RefPtr<Fontset> Context::load_fontset(const FontDescription& desc, PangoLanguage* language)
{
  return Glib::wrap(pango_context_load_fontset(gobj(), (desc).gobj(), language));
}

FontMetrics Context::get_metrics(const FontDescription& desc, const PangoLanguage* language)
{
  return FontMetrics(pango_context_get_metrics(gobj(), (desc).gobj(), const_cast<PangoLanguage*>(language)));
}

void Context::set_font_description(const FontDescription& desc)
{
  pango_context_set_font_description(gobj(), (desc).gobj());
}

FontDescription Context::get_font_description()
{
  return FontDescription(pango_context_get_font_description(gobj()));
}

PangoLanguage* Context::get_language()
{
  return pango_context_get_language(gobj());
}

void Context::set_language(const PangoLanguage* language)
{
  pango_context_set_language(gobj(), const_cast<PangoLanguage*>(language));
}

void Context::set_base_dir(Direction direction)
{
  pango_context_set_base_dir(gobj(), ((PangoDirection)(direction)));
}

Direction Context::get_base_dir()
{
  return ((Direction)(pango_context_get_base_dir(gobj())));
}


} // namespace Pango

