
#include <pangomm/fontmetrics.h>
#include <pangomm/private/fontmetrics_p.h>

// -*- c++ -*-
/* $Id: fontmetrics.ccg,v 1.1 2001/10/22 05:29:24 murrayc Exp $ */

/*
 *
 * Copyright 1998-1999 The Gtk-- Development Team
 * Copyright 2001      Free Software Foundation
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


namespace
{
} // anonymous namespace


namespace Glib
{

Pango::FontMetrics wrap(PangoFontMetrics* object, bool take_copy /* = false */)
{
  return Pango::FontMetrics(object, take_copy);
}

} // namespace Glib


namespace Pango
{


// static
GType FontMetrics::get_type()
{
  return pango_font_metrics_get_type();
}

FontMetrics::FontMetrics()
:
  gobject_ (0) // Allows creation of invalid wrapper, e.g. for output arguments to methods.
{}

FontMetrics::FontMetrics(const FontMetrics& src)
:
  gobject_ ((src.gobject_) ? pango_font_metrics_ref(src.gobject_) : 0)
{}

FontMetrics::FontMetrics(PangoFontMetrics* castitem, bool make_a_copy /* = true */)
{
  // For BoxedType wrappers, make_a_copy is true by default.  The static
  // BoxedType wrappers must always take a copy, thus make_a_copy = true
  // ensures identical behaviour if the default argument is used.

  if(make_a_copy)
  {
    if(castitem)
      gobject_ = pango_font_metrics_ref(castitem);
    else
      gobject_ = 0;
  }
  else
  {
    // It was given to us by a function which has already made a copy for us to keep.
    gobject_ = castitem;
  }
}

FontMetrics& FontMetrics::operator=(const FontMetrics& src)
{
  PangoFontMetrics *const new_gobject = (src.gobject_) ? pango_font_metrics_ref(src.gobject_) : 0;

  if(gobject_)
    pango_font_metrics_unref(gobject_);
  gobject_ = new_gobject;

  return *this;
}

FontMetrics::~FontMetrics()
{
  if(gobject_)
    pango_font_metrics_unref(gobject_);}


PangoFontMetrics* FontMetrics::gobj_copy() const
{
  return pango_font_metrics_ref(gobject_);
}

int FontMetrics::get_ascent()
{
  return pango_font_metrics_get_ascent(gobj());
}

int FontMetrics::get_descent()
{
  return pango_font_metrics_get_descent(gobj());
}

int FontMetrics::get_approximate_char_width()
{
  return pango_font_metrics_get_approximate_char_width(gobj());
}

int FontMetrics::get_approximate_digit_width()
{
  return pango_font_metrics_get_approximate_digit_width(gobj());
}


} // namespace Pango


